@@ -5,6 +5,8 @@ var NAVTREE =
       [ "Introduction to the SoapClient Module", "index.html#soapclientintro", null ],
       [ "Example", "index.html#Example", null ],
       [ "SoapClient Release Notes", "index.html#soapclientrelnotes", [
+        [ "SoapClient v0.2.5", "index.html#soapclient_0_2_5", null ],
+        [ "SoapClient v0.2.4.1", "index.html#soapclient_0_2_4_1", null ],
         [ "SoapClient v0.2.4", "index.html#soapclient_0_2_4", null ],
         [ "SoapClient v0.2.3", "index.html#soapclient_0_2_3", null ]
       ] ]
