@@ -1,6 +1,8 @@
 var searchData=
 [
   ['hasattributes',['hasAttributes',['../../../xml/html/classQore_1_1Xml_1_1XmlReader.html#a0243e7faeb87a1bbee4d94a08f5bccaa',1,'Qore::Xml::XmlReader']]],
+  ['hasfault',['hasFault',['../../../WSDL/html/class_w_s_d_l_1_1_w_s_operation.html#abc498e5b4704469e1843a0e106bee5b3',1,'WSDL::WSOperation']]],
+  ['hasheaders',['hasHeaders',['../../../WSDL/html/class_w_s_d_l_1_1_binding_message_description.html#ac38929552d5ef97c2c2876b74252927a',1,'WSDL::BindingMessageDescription']]],
   ['hassoap11',['hasSoap11',['../../../WSDL/html/class_w_s_d_l_1_1_namespaces.html#ade5f5a0e41f23082c4415000c3b66f92',1,'WSDL::Namespaces']]],
   ['hassoap12',['hasSoap12',['../../../WSDL/html/class_w_s_d_l_1_1_namespaces.html#a381ffac3e3dfc35d25a1f08bc3977a64',1,'WSDL::Namespaces']]],
   ['hasvalue',['hasValue',['../../../xml/html/classQore_1_1Xml_1_1XmlReader.html#aa9030809f95460d3861fa2a8b0c19fce',1,'Qore::Xml::XmlReader']]]
