@@ -2,5 +2,6 @@ var annotated_dup =
 [
     [ "Qore", null, [
       [ "Xml", "namespaceQore_1_1Xml.html", "namespaceQore_1_1Xml" ]
-    ] ]
+    ] ],
+    [ "MakeXmlOpts", "classMakeXmlOpts.html", "classMakeXmlOpts" ]
 ];
\ No newline at end of file
