@@ -1,6 +1,8 @@
 var namespaceQore_1_1Xml =
 [
+    [ "AbstractXmlIoInputCallback", "classQore_1_1Xml_1_1AbstractXmlIoInputCallback.html", "classQore_1_1Xml_1_1AbstractXmlIoInputCallback" ],
     [ "FileSaxIterator", "classQore_1_1Xml_1_1FileSaxIterator.html", "classQore_1_1Xml_1_1FileSaxIterator" ],
+    [ "InputStreamSaxIterator", "classQore_1_1Xml_1_1InputStreamSaxIterator.html", "classQore_1_1Xml_1_1InputStreamSaxIterator" ],
     [ "SaxIterator", "classQore_1_1Xml_1_1SaxIterator.html", "classQore_1_1Xml_1_1SaxIterator" ],
     [ "XmlDoc", "classQore_1_1Xml_1_1XmlDoc.html", "classQore_1_1Xml_1_1XmlDoc" ],
     [ "XmlNode", "classQore_1_1Xml_1_1XmlNode.html", "classQore_1_1Xml_1_1XmlNode" ],
