@@ -1,5 +1,7 @@
 var searchData=
 [
+  ['inputstreamsaxiterator',['InputStreamSaxIterator',['../classQore_1_1Xml_1_1InputStreamSaxIterator.html',1,'Qore::Xml']]],
+  ['invalidhash',['InvalidHash',['../classMakeXmlOpts_1_1InvalidHash.html',1,'MakeXmlOpts']]],
   ['isblank',['isBlank',['../classQore_1_1Xml_1_1XmlNode.html#a03ce057fda8f7e3479bc33151ed8fc8e',1,'Qore::Xml::XmlNode']]],
   ['isdefault',['isDefault',['../classQore_1_1Xml_1_1XmlReader.html#ab17ff2c041904d2ab9d6d464169c8a76',1,'Qore::Xml::XmlReader']]],
   ['isemptyelement',['isEmptyElement',['../classQore_1_1Xml_1_1XmlReader.html#a615f6a212ccb797ea699f4db7ac28b9a',1,'Qore::Xml::XmlReader']]],
