@@ -4,7 +4,7 @@
 
  Qore Programming Language
 
- Copyright (C) 2003 - 2014 David Nichols
+ Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -26,123 +26,206 @@
 
 #include <libxml/xmlreader.h>
 
+#include "qore/InputStream.h"
+
+#include "qore-xml-module.h"
 #include "QoreXmlDoc.h"
+#include "QC_AbstractXmlIoInputCallback.h"
 
 #include <errno.h>
 
 // FIXME: need to make error reporting consistent and set ExceptionSink for each call, not in constructor and then fix ql_xml.cc and adjust QC_XmlReader.cc
 
+class XmlIoInputCallbackHelper {
+public:
+    DLLLOCAL XmlIoInputCallbackHelper(const QoreHashNode* opts, ExceptionSink* xs) : xsink(xs) {
+        assert(!xml_io_callback);
+
+        bool found = false;
+        QoreValue v = opts->getValueKeyValueExistence("xml_input_io", found);
+        if (found) {
+            if (v.getType() != NT_OBJECT) {
+                xsink->raiseException("XMLREADER-XSD-ERROR", "expecting type 'object' with option 'xml_input_io'; got type '%s' instead", v.getTypeName());
+                return;
+            }
+            const QoreObject* obj = v.get<const QoreObject>();
+            xml_io_callback = static_cast<AbstractXmlIoInputCallback*>(obj->getReferencedPrivateData(CID_ABSTRACTXMLIOINPUTCALLBACK, xsink));
+            if (*xsink) {
+                assert(!xml_io_callback);
+                return;
+            }
+            if (!xml_io_callback) {
+                assert(!*xsink);
+                xsink->raiseException("XMLREADER-XSD-ERROR", "expecting an object of class 'AbstractXmlIoInputCallback' with option 'xml_input_io'; got class '%s' instead", obj->getClassName());
+                return;
+            }
+            xml_io_callback->setExceptionContext(xsink);
+        }
+    }
+
+    DLLLOCAL ~XmlIoInputCallbackHelper() {
+        if (xml_io_callback) {
+            xml_io_callback->clearExceptionContext();
+            xml_io_callback->deref(xsink);
+            xml_io_callback = nullptr;
+        }
+    }
+
+private:
+    ExceptionSink* xsink;
+};
+
 class QoreXmlReader {
 protected:
-   xmlTextReader* reader;
-   const QoreString* xml;
-   ExceptionSink* xs;
-   int fd;
+   xmlTextReader* reader = nullptr;
+   const QoreString* xml = nullptr;
+   ExceptionSink* xs = nullptr;
+   int fd = -1;
+   ReferenceHolder<InputStream> inputStream;
+   AbstractXmlValidator* val = nullptr;
 
    static void qore_xml_error_func(QoreXmlReader* xr, const char* msg, xmlParserSeverities severity, xmlTextReaderLocatorPtr locator) {
       if (severity == XML_PARSER_SEVERITY_VALIDITY_WARNING
-	  || severity == XML_PARSER_SEVERITY_WARNING) {
-	 printd(1, "XML parser warning: %s", msg);
-	 return;
+          || severity == XML_PARSER_SEVERITY_WARNING) {
+         printd(1, "XML parser warning: %s\n", msg);
+         return;
       }
       if (!xr->xs)
-	 return;
+         return;
       if (*(xr->xs))
-	 return;
+         return;
       QoreStringNode* desc = new QoreStringNode(msg);
       desc->chomp();
       xr->xs->raiseException("PARSE-XML-EXCEPTION", desc);
    }
 
-   DLLLOCAL void setExceptionSink(ExceptionSink* xsink) {
-      assert((!xsink && xs) || (xsink && !xs));
-      xs = xsink;
+   static int streamReadCallback(void* context, char* buffer, int len) {
+      QoreXmlReader *xmlReader = static_cast<QoreXmlReader*>(context);
+      int64 i = xmlReader->inputStream->read(buffer, len, xmlReader->xs);
+      if (*xmlReader->xs) {
+         return -1;
+      }
+      return i;
+   }
+
+   static int streamCloseCallback(void* context) {
+      return 0;
    }
 
    DLLLOCAL AbstractQoreNode* getXmlData(ExceptionSink* xsink, const QoreEncoding* data_ccsid, int pflags = XPF_NONE, int min_depth = -1);
 
-   DLLLOCAL void init(const QoreString* n_xml, int options, ExceptionSink* xsink) {
+   DLLLOCAL void init(const char* enc, int options, const QoreHashNode* opts, ExceptionSink* xsink) {
+      assert(!xml);
+      assert(!reader);
+      xs = xsink;
+      reader = xmlReaderForIO(streamReadCallback, streamCloseCallback, this, 0, enc, options);
+      if (!reader) {
+         xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
+         return;
+      }
+
+      xmlTextReaderSetErrorHandler(reader, (xmlTextReaderErrorFunc)qore_xml_error_func, this);
+
+      if (opts)
+         processOpts(opts, xsink);
+      //printd(5, "QoreXmlReader::init() valid: %d\n", isValid());
+   }
+
+   DLLLOCAL void init(const QoreString* n_xml, int options, const QoreHashNode* opts, ExceptionSink* xsink) {
+      assert(!xml);
+      assert(!reader);
       xml = n_xml;
 
       assert(xml->getEncoding() == QCS_UTF8);
       reader = xmlReaderForDoc((xmlChar*)xml->getBuffer(), 0, 0, options);
       if (!reader) {
-	 xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
-	 return;
+         xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
+         return;
       }
 
       xmlTextReaderSetErrorHandler(reader, (xmlTextReaderErrorFunc)qore_xml_error_func, this);
+      //printd(5, "QoreXmlReader::init() xml size: %d opts: %p reader: %p set error handler; options: %d\n", (int)xml->size(), opts, reader, options);
+
+      if (opts)
+         processOpts(opts, xsink);
    }
 
+   DLLLOCAL void processOpts(const QoreHashNode* opts, ExceptionSink* xsink);
+
    DLLLOCAL void init(xmlDocPtr doc, ExceptionSink* xsink) {
-      xml = 0;
+      assert(!xml);
+      assert(!reader);
       reader = xmlReaderWalker(doc);
       if (!reader) {
-	 xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
-	 return;
+         xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
+         return;
       }
       // the following call causes a crash - I guess the document has already been parsed anyway
       //xmlTextReaderSetErrorHandler(reader, (xmlTextReaderErrorFunc)qore_xml_error_func, xsink);
    }
 
-   DLLLOCAL void init(ExceptionSink* xsink, const char* fn, const char* encoding, int options) {
-      xml = 0;
+   DLLLOCAL void init(ExceptionSink* xsink, const char* fn, const char* encoding, int options, const QoreHashNode* opts = nullptr) {
+      assert(!xml);
+      assert(!reader);
+      assert(fd == -1);
       fd = open(fn, O_RDONLY);
       if (fd < 0) {
          reader = 0;
          xsink->raiseErrnoException("XML-READER-ERROR", errno, "could not open '%s' for reading", fn);
          return;
       }
+
       reader = xmlReaderForFd(fd, 0, encoding, options);
       if (!reader) {
          close(fd);
-	 xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
-	 return;
+         fd = -1;
+         xsink->raiseException("XML-READER-ERROR", "could not create XML reader");
+         return;
       }
 
       xmlTextReaderSetErrorHandler(reader, (xmlTextReaderErrorFunc)qore_xml_error_func, this);
+      //printd(5, "QoreXmlReader::init() opts: %p reader: %p set error handler\n", opts, reader);
+
+      if (opts)
+         processOpts(opts, xsink);
    }
 
    DLLLOCAL int do_int_rv(int rc, ExceptionSink* xsink) {
       if (rc == -1 && !*xsink)
-	 xsink->raiseExceptionArg("PARSE-XML-EXCEPTION", xml ? new QoreStringNode(*xml) : 0, "error parsing XML string");
+         xsink->raiseExceptionArg("PARSE-XML-EXCEPTION", xml ? new QoreStringNode(*xml) : 0, "error parsing XML string");
       return rc;
    }
 
-   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, const QoreString* n_xml, int options) : xs(0), fd(-1) {
-      init(n_xml, options, xsink);
+   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, InputStream *is, const char* enc, int options, const QoreHashNode* opts) : inputStream(is, xsink) {
+      init(enc, options, opts, xsink);
    }
 
-   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, xmlDocPtr doc) : xs(0), fd(-1) {
+   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, const QoreString* n_xml, int options, const QoreHashNode* opts = nullptr) : inputStream(xsink) {
+      init(n_xml, options, opts, xsink);
+   }
+
+   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, xmlDocPtr doc) : inputStream(xsink) {
       init(doc, xsink);
    }
 
-   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, const QoreString* n_xml, int options, xmlDocPtr doc, const char* fn, const char* enc) : xs(0), fd(-1) {
+   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, const QoreString* n_xml, int options, xmlDocPtr doc, const char* fn, const char* enc) : inputStream(xsink) {
       if (fn)
          init(xsink, fn, enc, options);
       else
          init(xsink, n_xml, options, doc);
    }
 
-   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, const char* fn, const char* encoding, int options) : xs(0), fd(-1) {
-      init(xsink, fn, encoding, options);
+   DLLLOCAL QoreXmlReader(ExceptionSink* xsink, const char* fn, const char* encoding, int options, const QoreHashNode* opts) : inputStream(xsink) {
+      init(xsink, fn, encoding, options, opts);
    }
 
    DLLLOCAL void reset(ExceptionSink* xsink, const QoreString* n_xml, int options, xmlDocPtr doc) {
-      if (reader) {
-	 xmlFreeTextReader(reader);
-         reader = 0;
-      }
+      reset();
       init(xsink, n_xml, options, doc);
    }
 
    DLLLOCAL void reset(ExceptionSink* xsink, const char* fn, const char* enc, int options) {
-      if (reader) {
-	 xmlFreeTextReader(reader);
-         reader = 0;
-      }
-      if (fd >= 0)
-         close(fd);
+      reset();
       init(xsink, fn, enc, options);
    }
 
@@ -151,7 +234,7 @@ protected:
 
       if (n_xml) {
          assert(!doc);
-         init(n_xml, options, xsink);
+         init(n_xml, options, nullptr, xsink);
       }
       else {
          assert(!n_xml);
@@ -159,84 +242,115 @@ protected:
       }
    }
 
+    DLLLOCAL void reset() {
+        //printd(5, "QoreXmlReader::reset() reader: %p val: %p fd: %d\n", reader, val, fd);
+        if (val) {
+            delete val;
+            val = nullptr;
+        }
+        if (reader) {
+            xmlFreeTextReader(reader);
+            reader = nullptr;
+        }
+        if (fd >= 0) {
+            close(fd);
+            fd = -1;
+        }
+        if (xml)
+            xml = nullptr;
+    }
+
 public:
-   DLLLOCAL QoreXmlReader(const QoreString* n_xml, int options, ExceptionSink* xsink) : xs(xsink), fd(-1) {
-      init(n_xml, options, xsink);
+   DLLLOCAL QoreXmlReader(const QoreString* n_xml, int options, ExceptionSink* xsink) : xs(xsink), inputStream(xsink) {
+      init(n_xml, options, nullptr, xsink);
    }
 
-   DLLLOCAL QoreXmlReader(xmlDocPtr doc, ExceptionSink* xsink) : xs(xsink), fd(-1) {
+   DLLLOCAL QoreXmlReader(xmlDocPtr doc, ExceptionSink* xsink) : xs(xsink), inputStream(xsink) {
       init(doc, xsink);
    }
 
    DLLLOCAL ~QoreXmlReader() {
-      if (reader)
-	 xmlFreeTextReader(reader);
-      if (fd >= 0)
-         close(fd);
+      reset();
    }
 
    DLLLOCAL operator bool() const {
       return reader != 0;
    }
 
-   // returns 0=OK, -1=error
+   DLLLOCAL void setExceptionContext(ExceptionSink* xsink) {
+      if (xs != xsink)
+         xs = xsink;
+      if (val)
+         val->setExceptionContext(xsink);
+   }
+
+   // returns 1 = OK, 0 = no more nodes to read, -1 = error
    DLLLOCAL int read(ExceptionSink* xsink) {
+      setExceptionContext(xsink);
+
       int rc = read();
       if (rc == -1) {
-	 if (!*xsink)
-	    xsink->raiseExceptionArg("PARSE-XML-EXCEPTION", xml ? new QoreStringNode(*xml) : 0, "cannot parse XML string");
+         if (!*xsink)
+            xsink->raiseExceptionArg("PARSE-XML-EXCEPTION", xml ? new QoreStringNode(*xml) : 0, "cannot parse XML string");
       }
       return rc;
    }
 
+   // returns 1 = OK, 0 = no more nodes to read, -1 = error
    DLLLOCAL int read(const char* info, ExceptionSink* xsink) {
+      setExceptionContext(xsink);
+
       int rc = read();
       if (rc == -1) {
-	 if (!*xsink)
-	    xsink->raiseExceptionArg("PARSE-XML-EXCEPTION", xml ? new QoreStringNode(*xml) : 0, "cannot parse XML string: %s", info);
+         if (!*xsink)
+            xsink->raiseExceptionArg("PARSE-XML-EXCEPTION", xml ? new QoreStringNode(*xml) : 0, "cannot parse XML string: %s", info);
       }
       return rc;
    }
 
+   // returns 1 = OK, 0 = no more nodes to read, -1 = error
    DLLLOCAL int read() {
       return xmlTextReaderRead(reader);
    }
 
+   // returns 1 = OK, 0 = no more nodes to read, -1 = error
    DLLLOCAL int readSkipWhitespace() {
       int rc;
       while (true) {
-	 rc = read();
-	 if (rc != 1)
-	    break;
- 	 int nt = xmlTextReaderNodeType(reader);
-	 if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
-	    break;
+         rc = read();
+         if (rc != 1)
+            break;
+         int nt = xmlTextReaderNodeType(reader);
+         if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
+            break;
       }
       return rc;
    }
 
+   // returns 1 = OK, 0 = no more nodes to read, -1 = error
    DLLLOCAL int readSkipWhitespace(ExceptionSink* xsink) {
       int rc;
       while (true) {
-	 rc = read(xsink);
-	 if (rc != 1)
-	    break;
- 	 int nt = xmlTextReaderNodeType(reader);
-	 if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
-	    break;
+         rc = read(xsink);
+         if (rc != 1)
+            break;
+         int nt = xmlTextReaderNodeType(reader);
+         if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
+            break;
       }
       return rc;
    }
 
+   // returns 1 = OK, 0 = no more nodes to read, -1 = error
    DLLLOCAL int readSkipWhitespace(const char* info, ExceptionSink* xsink) {
       int rc;
       while (true) {
-	 rc = read(info, xsink);
-	 if (rc != 1)
-	    break;
- 	 int nt = xmlTextReaderNodeType(reader);
-	 if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
-	    break;
+         rc = read(info, xsink);
+         if (rc != 1)
+            break;
+         int nt = xmlTextReaderNodeType(reader);
+         if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
+            break;
       }
       return rc;
    }
@@ -249,13 +363,13 @@ public:
    DLLLOCAL int nodeTypeSkipWhitespace() {
       int nt;
       while (true) {
-	 nt = xmlTextReaderNodeType(reader);
-	 if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
-	    break;
+         nt = xmlTextReaderNodeType(reader);
+         if (nt != XML_READER_TYPE_SIGNIFICANT_WHITESPACE)
+            break;
 
-	 // get next element
-	 if (read() != 1)
-	    return -1;
+         // get next element
+         if (read() != 1)
+            return -1;
       }
       return nt;
    }
@@ -294,7 +408,7 @@ public:
 #else
       xmlNodePtr node = xmlTextReaderCurrentNode(reader);
       if (!node)
-	 return false;
+         return false;
 
       return node->type == XML_NAMESPACE_DECL ? true : false;
 #endif
@@ -314,13 +428,14 @@ public:
 
    DLLLOCAL QoreStringNode* getValue(const QoreEncoding* id, ExceptionSink* xsink) {
       if (id == QCS_UTF8)
-	 return new QoreStringNode(constValue(), QCS_UTF8);
+         return new QoreStringNode(constValue(), QCS_UTF8);
 
       return QoreStringNode::createAndConvertEncoding(constValue(), QCS_UTF8, id, xsink);
    }
 
 #ifdef HAVE_XMLTEXTREADERSETSCHEMA
    DLLLOCAL int setSchema(xmlSchemaPtr schema) {
+      //printd(5, "QoreXmlReader::setSchema() reader: %p schema: %p\n", reader, schema);
       return xmlTextReaderSetSchema(reader, schema);
    }
 #endif
@@ -418,9 +533,10 @@ public:
    }
 
    DLLLOCAL int next(ExceptionSink* xsink) {
+      setExceptionContext(xsink);
       int rc = xmlTextReaderNext(reader);
       if (rc == -1 && !*xsink)
-	 xsink->raiseException("PARSE-XML-EXCEPTION", "error parsing XML string");
+         xsink->raiseException("PARSE-XML-EXCEPTION", "error parsing XML string");
       return rc;
    }
 
@@ -431,25 +547,27 @@ public:
    }
 */
 
-   DLLLOCAL QoreStringNode* getInnerXml() {
+   DLLLOCAL QoreStringNode* getInnerXml(ExceptionSink* xsink) {
+      setExceptionContext(xsink);
       return doString(xmlTextReaderReadInnerXml(reader));
    }
 
-   DLLLOCAL QoreStringNode* getOuterXml() {
+   DLLLOCAL QoreStringNode* getOuterXml(ExceptionSink* xsink) {
+      setExceptionContext(xsink);
       return doString(xmlTextReaderReadOuterXml(reader));
    }
 
 #ifdef HAVE_XMLTEXTREADERRELAXNGSETSCHEMA
    DLLLOCAL void relaxNGValidate(const char* rng, ExceptionSink* xsink) {
       if (xmlTextReaderRelaxNGValidate(reader, rng))
-	 xsink->raiseException("XMLREADER-RELAXNG-ERROR", "an error occured setting the RelaxNG schema for validation; this function must be called before the first call to XmlReader::read()");
+         xsink->raiseException("XMLREADER-RELAXNG-ERROR", "an error occurred setting the RelaxNG schema for validation; this function must be called before the first call to XmlReader::read()");
    }
 #endif
 
 #ifdef HAVE_XMLTEXTREADERSETSCHEMA
    DLLLOCAL void schemaValidate(const char* xsd, ExceptionSink* xsink) {
       if (xmlTextReaderSchemaValidate(reader, xsd))
-	 xsink->raiseException("XMLREADER-XSD-ERROR", "an error occured setting the W3C XSD schema for validation; this function must be called before the first call to XmlReader::read()");
+         xsink->raiseException("XMLREADER-XSD-ERROR", "an error occurred setting the W3C XSD schema for validation; this function must be called before the first call to XmlReader::read()");
    }
 #endif
 
