@@ -1,26 +1,26 @@
 /* -*- mode: c++; indent-tabs-mode: nil -*- */
 /*
-  ql_xml.h
+    ql_xml.h
 
-  libxml2-based XML functions
+    libxml2-based XML functions
 
-  Qore Programming Language
+    Qore Programming Language
 
-  Copyright 2003 - 2015 David Nichols
+    Copyright 2003 - 2017 Qore Technologies, s.r.o.
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
 
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef _QORE_QL_XML_H
@@ -31,9 +31,11 @@
 
 #include <qore/Qore.h>
 
+class MakeXmlOpts;
+
 DLLLOCAL void init_xml_constants(QoreNamespace& ns);
 
-DLLLOCAL QoreStringNode* make_xml(ExceptionSink* xsink, const QoreEncoding* enc, const QoreHashNode& h, int flags = XGF_NONE);
+DLLLOCAL QoreStringNode* make_xml(ExceptionSink* xsink, const QoreHashNode &h, const MakeXmlOpts &opts);
 DLLLOCAL QoreStringNode* make_xmlrpc_call(ExceptionSink* xsink, const QoreEncoding* ccs, int offset, const QoreValueList* args, int flags = 0);
 DLLLOCAL QoreStringNode* make_xmlrpc_call_args(ExceptionSink* xsink, const QoreEncoding* ccs, int offset, const QoreValueList* args, int flags = 0);
 // ccsid is the output encoding for strings
@@ -47,49 +49,35 @@ DLLLOCAL const char* get_xml_element_type_name(int t);
 DLLLOCAL const char* get_xml_node_type_name(int t);
 
 #ifdef HAVE_XMLTEXTREADERSETSCHEMA
-class QoreXmlSchemaContext {
-   friend class QoreXmlSchemaValidContext;
+class QoreXmlSchemaContext : public AbstractXmlValidator, Utf8StringHelper {
 protected:
-   xmlSchemaPtr schema;
-
-   DLLLOCAL xmlSchemaValidCtxtPtr getValidCtxtPtr() {
-      return xmlSchemaNewValidCtxt(schema);
-   }
+    xmlSchemaPtr schema = nullptr;
+    xmlSchemaValidCtxtPtr ctx = nullptr;
 
 public:
-   DLLLOCAL QoreXmlSchemaContext(const char* xsd, int size, ExceptionSink* xsink);
-   DLLLOCAL ~QoreXmlSchemaContext() {
-      if (schema)
-	 xmlSchemaFree(schema);
-   }
-   DLLLOCAL operator bool() const {
-      return schema != 0;
-   }
-   DLLLOCAL xmlSchemaPtr getSchema() {
-      return schema;
-   }
-};
+    DLLLOCAL QoreXmlSchemaContext(const QoreString& xsd, ExceptionSink* xsink);
 
-class QoreXmlSchemaValidContext {
-protected:
-   xmlSchemaValidCtxtPtr ptr;
+    DLLLOCAL virtual ~QoreXmlSchemaContext() {
+        if (ctx)
+            xmlSchemaFreeValidCtxt(ctx);
+        if (schema)
+            xmlSchemaFree(schema);
+    }
 
-   DLLLOCAL QoreXmlSchemaValidContext(xmlSchemaValidCtxtPtr n_ptr) : ptr(n_ptr) {
-   }
+    DLLLOCAL operator bool() const {
+        return schema != 0;
+    }
 
-public:
-   DLLLOCAL QoreXmlSchemaValidContext(QoreXmlSchemaContext& c) : ptr(c.getValidCtxtPtr()) {
-      assert(ptr);
-   }
-   DLLLOCAL ~QoreXmlSchemaValidContext() {
-      xmlSchemaFreeValidCtxt(ptr);
-   }
-   DLLLOCAL xmlSchemaValidCtxtPtr getPtr() {
-      return ptr;
-   }
-   DLLLOCAL int validateDoc(xmlDocPtr doc) {
-      return xmlSchemaValidateDoc(ptr, doc);
-   }
+    DLLLOCAL xmlSchemaPtr getSchema() {
+        return schema;
+    }
+
+    DLLLOCAL xmlSchemaValidCtxtPtr getPtr();
+
+    DLLLOCAL virtual int validateDoc(xmlDocPtr doc) {
+        getPtr();
+        return xmlSchemaValidateDoc(ctx, doc);
+    }
 };
 #endif
 
@@ -107,7 +95,7 @@ public:
    DLLLOCAL QoreXmlRelaxNGContext(const char* rng, int size, ExceptionSink* xsink);
    DLLLOCAL ~QoreXmlRelaxNGContext() {
       if (schema)
-	 xmlRelaxNGFree(schema);
+         xmlRelaxNGFree(schema);
    }
    DLLLOCAL operator bool() const {
       return schema != 0;
